@@ -1,6 +1,7 @@
 //go:build !plan9 && !js
 // +build !plan9,!js
 
+// Package kv provides key/value database.
 package kv
 
 import (
@@ -215,7 +216,7 @@ func (db *DB) loop() {
 
 // Do a key-value operation and return error when done
 func (db *DB) Do(write bool, op Op) error {
-	if db.queue == nil {
+	if db == nil || db.queue == nil {
 		return ErrInactive
 	}
 	r := &request{
@@ -292,6 +293,11 @@ func (db *DB) Stop(remove bool) error {
 	return db.Do(false, &opStop{remove: remove})
 }
 
+// IsStopped returns true if db is already stopped
+func (db *DB) IsStopped() bool {
+	return len(dbMap) == 0
+}
+
 // opStop: close database and stop operation loop
 type opStop struct {
 	remove bool
